/*
1. Compiler: TC 2.0
2. Mode: Small
3. Project: Echocom1.prj
   	    Echocom1.c
	    I7188s.lib
4. Explain: Creating a connection from pc and 7188 to control the 7060.
            * Press "*bBAUD" to change baudrate
              Press 'q'(not 'Q') to quit.
	      Press 's'(not 'S') to toggle checksum mode.
5. Hordware: 7188
It's compiled by Tony
---------------------------------------------------------------------------
*/

#include<stdlib.h>
#include<stdio.h>
#include<conio.h>
#include<dos.h>
#include<string.h>
#include"..\lib\i7188.h"
/*-------------------------------------------------------------------------*/

unsigned no;
unsigned char buf[80];
unsigned char OutBuf[80],OutCmd[80]={0};
int idx=0,LastIdx=0;
int outidx=0;
unsigned long far * const TimeTick=(unsigned long far *)0x0040006CL;

void ShowHelp(void)
{
  Puts("\n\rCommand line option:");
  Puts("\n\r/bxxxx : set baudrate to xxxx");
  Puts("\n\r/cx    : set checksum enable(x=1) or Disable(x=0)");
  Puts("\n\r--------------------------------------------------");
  Puts("\n\roperation Command:");
  Puts("\n\r/bxxxx : change baudrate to xxxx");
  Puts("\n\r/cx    : set checksum enable(x=1) or Disable(x=0)");
  Puts("\n\r/a     : scan 7000 modules use current setting");
  Puts("\n\r/exxx  : Set end scan address");
  Puts("\n\r/sxxx  : Set start scan address");
  Puts("\n\rOthers cmd will send to Com1");
  Puts("\n\rJust press ENTER will repeat last command\n\r");
}

void main(int argc,char *argv[])
{ long baud=9600,newbaud,timeout=48;
  int quit=0;
  int data;
  int i;
  int checksum=0;
  int fSearch=0;
  int Addr,EndAddr=255,StartAddr=0;
  int ShowPrompt=1;
  unsigned wait=0,wait1=0;
  unsigned long st;
  char *Prompt="Echo485>";

  if(!Is7188()){		/*detect the current operation system*/
     Print("\n\rEcho485.exe must run on I-7188!");
     return;
  }


  for(i=1;i<argc;i++){
     if(argv[i][0]=='/'){
       switch(argv[i][1]){
	case 'b': case 'B':
	     baud=atol(&argv[i][2]);
	     if(baud<=0) baud=9600;
	     break;
	case 's': case 'S':
	     checksum=atoi(&argv[i][2]);
	     break;
	case 't': case 'T':
	     timeout=atol(&argv[i][2]);
	     if(timeout<2) timeout=2;
	     break;
       }
     }
  }
  switch(baud){
     case 115200: timeout=4;	break;
     case 57600:  timeout=8;	break;
     case 38400:  timeout=12;	break;
     case 19200:  timeout=24;	break;
     case 9600:   timeout=48;	break;
     case 4800:   timeout=96;	break;
     case 2400:   timeout=192;	break;
     case 1200:   timeout=384;	break;
  }


  for(i=0;i<argc;i++){
      Print("Argv[%d]=%s addr=%p\n\r",i,argv[i],argv[i]);
  }
  Print("\n\rEcho485 Ver. 2.00 for I-7188(D)");
  Print("\n\rUse Com1(RS485) connect to I-7000 series");
  Print("\n\recho485 [/option]");
  Print("\n\r/? Show help");
  Print("\n\r/q to quit to ROM-DOS");
  Print("\n\rCurrent mode:baudrate=%ld,checksum=%d,timeout=%ld\n\r",baud,checksum,timeout);

  InstallCom1(baud,8,0,1);

  Init5DigitLed();
  Show5DigitLed(1,14);
  Show5DigitLed(2,12);
  Show5DigitLed(3,4);
  Show5DigitLed(4,8);
  Show5DigitLed(5,5);
  st=*TimeTick;
  OutBuf[0]=0;
  while(!quit){
     if(ShowPrompt){ ShowPrompt=0; Puts(Prompt); }
     if(fSearch){
	if(!wait){
	   sprintf(OutBuf,"$%02XM",Addr);
	   outidx=4;
	   wait=1;
	   Print("\rAddr=%02X-->",Addr);
	   goto SendCmd;
	}
	else {
	   if((*TimeTick-st)>=timeout) {
	      wait=0;
	      Addr++;
	      if(Addr>EndAddr){
		 fSearch=0;
		 Print("\n\r");
		 ShowPrompt=1;
	      }
	   }
	}
     }
     if(Kbhit()){
	data=Getch();
	if(data=='\b'){
	      if(outidx){
		 outidx--;
		 Putch(data);
	      }
	}
	else {
	   Putch(data);
	   if(data=='\r'){
	      unsigned char sum;
	      ClearCom1();
	      Putch('\n');
	      if(outidx) OutBuf[outidx]=0;

	      if(outidx && OutBuf[0]=='/'){
		 switch(OutBuf[1]){
		   case 'b':  case 'B':
			newbaud=atol(OutBuf+2);
			if(NoError==SetBaudrate2(newbaud)) {
			   baud=newbaud;
			   Print("Change baudrate to %ld\n\r",baud);
			}
			break;
		   case 'c':  case 'C':
			checksum=atoi(OutBuf+2);
			Print("CheckSum %s\n\r",checksum ? "Enable":"Disable");
			break;
		   case 'a':  case 'A':
			fSearch=1;
			Addr=StartAddr;
			Print("Scan from Addr %d(%02X) to %d(%02X)\n\r",StartAddr,StartAddr,EndAddr,EndAddr);
			break;
		   case 'e':  case 'E':
			EndAddr=atoi(OutBuf+2);
			EndAddr &= 0xFF;
			if(EndAddr<StartAddr) EndAddr=StartAddr;
			Print("Set End scan address to %d(%02X)\n\r",EndAddr,EndAddr);
			break;
		   case 's':  case 'S':
			StartAddr=atoi(OutBuf+2);
			StartAddr &= 0xFF;
			if(StartAddr>EndAddr) StartAddr=EndAddr;
			Print("Set Start scan address to %d(%02X)\n\r",StartAddr,StartAddr);
			break;
		   case 'h':
			{ extern unsigned long Com4InterruptCount;
			  Print("Count=%lu\n\r",Com4InterruptCount);
			}
			break;
		   case 't':  case 'T':
			timeout=atol(OutBuf+2);
			if(timeout<2) timeout=2;
			Print("Set timeout=%ld\n\r",timeout);
			break;

		   case '?':
			ShowHelp();
			break;
		   case 'q':  case 'Q':
			quit=1;
			break;
		   default:
			Print("Unknown command\n\r");
			break;
		 }
		 outidx=0;
		 ShowPrompt=1;
	      }
	      else {
SendCmd:
		 if(outidx){
		   if(!wait){
		     LastIdx=outidx;
		     strcpy(OutCmd,OutBuf);
		   }
		 }
		 else {
		   outidx=LastIdx;
		   strcpy(OutBuf,OutCmd);
		 }
		 Print("[%s",OutBuf);
		 sum=0;
		 for(i=0;i<outidx;i++){
		   ToCom1(OutBuf[i]);
		   if(checksum) sum+=OutBuf[i];
		 }
		 if(checksum){
		   ToCom1((i=hex_to_ascii[sum>>4]));
		   Putch(i);
		   ToCom1((i=hex_to_ascii[sum&0xf]));
		   Putch(i);
		 }
		 ToCom1('\r');
		 Puts("]-->");
		 outidx=0;
		 st=*TimeTick;
		 if(!wait) wait1=1;
	      }
	   }
	   else {
	      OutBuf[outidx++]=data;
	   }
	}
     }
     if(IsCom1()){
	data=ReadCom1();
	if(data=='\r'){
	   buf[idx]=0;
	   wait1=0;
	   Print(" {%lu ms}",*TimeTick-st);
	   if(checksum){
	      unsigned char sum=0;
	      int i;
	      for(i=0;i<idx-2;i++)
		 sum+=buf[i];
	      if(buf[i] != hex_to_ascii[sum>>4] ||
		 buf[i+1] != hex_to_ascii[sum&0x0f]) {
		 Print(" Check sum error,must be %02X",sum);
	      }
	   }
	   idx=0;
	   if(wait){
	      Print(" wait=%u",wait);
	      wait=0;
	      Addr++;
	      if(Addr>EndAddr){
		 fSearch=0;
		 Print("\n\r");
		 ShowPrompt=1;
	      }
	   }
	   else ShowPrompt=1;
	   Putch(data);
	   Putch('\n');
	}
	else {
	   Putch(data);
	   buf[idx++]=data;
	   if(idx>=79) idx=78;
	}
     }
     else if(wait1 && (*TimeTick-st)>=timeout){
	     Print("{timeout!}\n\r");
	     ShowPrompt=1;
	     wait1=0;
     }
  }
  RestoreCom1();
}
